<?php
/**
 * Email class.
 * This handles custom WooCommerce emails for different FBA status.
 *
 * @package NeverSettle\WooCommerce-Amazon-Fulfillment
 * @since 4.4.0
 */

defined( 'ABSPATH' ) || exit;

if ( ! class_exists( 'NS_MCF_Email' ) ) {

	class NS_MCF_Email extends NS_MCF_Integration {

		/**
		 * Initialize class.
		 */
		public function init() {
			add_filter( 'woocommerce_email_classes', array( $this, 'add_email_class' ) );

			add_filter( 'woocommerce_locate_core_template', array( $this, 'locate_core_template' ), 10, 4 );
			add_filter( 'woocommerce_locate_template', array( $this, 'locate_template' ), 10, 3 );

			add_filter( 'woocommerce_email_actions', array( $this, 'add_email_action' ) );
		}

		/**
		 * Add a custom email to the list of emails WooCommerce should load.
		 *
		 * @param array $email_classes available email classes.
		 *
		 * @return array filtered available email classes.
		 */
		public function add_email_class( $email_classes ) {
			// include our custom email class
			require 'emails/class-ns-mcf-fba-order-email.php';

			// add the email class to the list of email classes that WooCommerce loads
			$email_classes['NS_MCF_FBA_Order_Email'] = new NS_MCF_FBA_Order_Email();

			return $email_classes;
		}

		/**
		 * Set the template path and load the template files.
		 *
		 * @param string $core_file The core template file to load.
		 * @param string $template The template name.
		 * @param string $template_base The template base.
		 * @param string $email_id The email id.
		 *
		 * @return string $core_file The core file.
		 */
		public function locate_core_template( $core_file, $template, $template_base, $email_id ) {
			$templates = array(
				'emails/customer-sent-to-fba-order.php',
				'emails/plain/customer-sent-to-fba-order.php',
			);

			if ( in_array( $template, $templates, true ) ) {
				return $this->ns_fba->plugin_path . $template;
			}

			return $core_file;
		}

		/**
		 * Load the template.
		 *
		 * @param string $template The template file.
		 * @param string $template_name The template name.
		 * @param string $template_path The template path.
		 *
		 * @return string $template The template.
		 */
		public function locate_template( $template, $template_name, $template_path ) {
			$templates = array(
				'emails/customer-sent-to-fba-order.php',
				'emails/plain/customer-sent-to-fba-order.php',
			);

			if ( in_array( $template_name, $templates, true ) ) {
				return $this->ns_fba->plugin_path . $template_name;
			}

			return $template;
		}

		/**
		 * Register the email action notification
		 *
		 * @param array $actions The email notification actions.
		 *
		 * @return array $actions Modified actions.
		 */
		public function add_email_action( $actions ) {
			if ( ! isset( $actions['woocommerce_order_status_processing_to_sent-to-fba'] ) ) {
				array_push( $actions, 'woocommerce_order_status_processing_to_sent-to-fba' );
				array_push( $actions, 'woocommerce_order_status_processing_to_part-to-fba' );
			}
			return $actions;
		}
	}
}
