<?php
/**
 * Utility class to admin notifications
 */

defined( 'ABSPATH' ) || exit;

if ( ! class_exists( 'NS_MCF_Notification' ) ) {

	class NS_MCF_Notification {

		/**
		 * Add admin notices as needed for notifications.
		 *
		 * @return bool
		 */
		public static function should_show_notification( $option_name = '_ns_mcf_notification' ) {
			if ( ! current_user_can( 'administrator' ) ) {
				return false;
			}

			$user_id      = get_current_user_id();
			$notification = self::get_user_meta( $user_id, $option_name );

			if ( ! $notification || 'yes' !== $notification ) {
				return true;
			}
			return false;
		}

		/**
		 * Save the request to hide the notification.
		 */
		public static function process_notification( $option_name = '_ns_mcf_notification' ) {
			$user_id = get_current_user_id();
			self::update_user_meta( $user_id, $option_name, 'yes' );
		}

		/**
		 * Update user meta.
		 *
		 * @param int   $user_id The user id.
		 * @param array $notification The notification.
		 */
		private static function update_user_meta( $user_id, $option_name, $notification ) {
			update_user_meta( $user_id, $option_name, $notification );
		}

		/**
		 * Get user meta.
		 *
		 * @param int $user_id The user id.
		 *
		 * @return bool|array
		 */
		private static function get_user_meta( $user_id, $option_name ) {
			return get_user_meta( $user_id, $option_name, true );
		}
	}
}
